/*****************************************************************************/
/* UNDDI.CPP   UnPack a .DDI format file                         <TC++\BC++> */
/* Version 0.01                   Jul,1994                    Lee Dong-Liang */
/*****************************************************************************/

#include <io.h>
#include <bios.h>
#include <dos.h>
#include <dir.h>
#include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <conio.h>
#include <string.h>
#include <fcntl.h>
#include <sys\stat.h>

#define  N_A            '\x61\x1E'
#define  N_O            '\x6F\x18'
#define  N_S            '\x73\x1F'
#define  S_A            '\x41\x1E'
#define  S_O            '\x4F\x18'
#define  S_S            '\x53\x1F'

extern unsigned _stklen=4096*8;        /* Length of Stack */
int *FATchain,totalFAT;                /* FATchain & totalFAT */
FILE *ddifile;                         /* File pointer of source DDI file */
char *DDIfilename;                     /* File name of source DDI file */
char *CurrentPATH;                     /* Current Target PATH */
char *CurrentPATHfilename;             /* Current Target PATH filename */
char *filebuffer;                      /* file buffer                  */
char FlagofOverwrite;                  /* flag of overwrite            */
char FlagofCheckonly;                  /* flag of check only           */
char FlagofNocmpFAT;                   /* flag of No to compare FAT    */
char FlagofStdformat;                  /* flag of use standard format  */
char DDIDisksize;                      /* flag of ddi diskfile specify */
char inchinesesystem;                  /* flag of in Chinese System */

/*------------------------------- .DDI format -------------------------------*/
enum DISKTYPE { Disk_unknow, Disk_360, Disk_1200, Disk_720, Disk_1440,
                Disk_2880, Disk_320 };

DISKTYPE ddifiletype;
unsigned int   BytePerSector;          /* bytes per sector                   */
unsigned char  SectorPerCluster;       /* allocation unit size (in sector)   */
unsigned int   ReservedSector;         /* reserved sectors (starting at 0)   */
unsigned char  FATnumber;              /* number of FAT's on disk            */
unsigned int   ROOTEntry;              /* number of root directory entries   */
unsigned int   TotalSector;            /* total sectors(0 if partition>32Mb) */
unsigned int   SectorPerFAT;           /* sectors per FAT                    */
unsigned int   SectorPerTrack;         /* sectors per track        (DOS 3.0+)*/
DISKTYPE GetDDIfiletype(void)
{
   DISKTYPE d;
   fseek(ddifile,10L,SEEK_SET);
   fread(&d,sizeof(DISKTYPE),1,ddifile);
   return d;
}
void Icannothandle(void)
{
   cout << " I cannot handle the file ! :< " << endl;
   exit(1);
}
void SetDefault(DISKTYPE d)
{
   switch(d)
   {
      case Disk_360    :   BytePerSector   =512;      SectorPerCluster=2;
                           ReservedSector  =1;        FATnumber       =2;
                           ROOTEntry       =112;      TotalSector     =720;
                           SectorPerFAT    =2;        SectorPerTrack  =9;
                           break;
      case Disk_1200   :   BytePerSector   =512;      SectorPerCluster=1;
                           ReservedSector  =1;        FATnumber       =2;
                           ROOTEntry       =224;      TotalSector     =2399;
                           SectorPerFAT    =7;        SectorPerTrack  =15;
                           break;
      case Disk_720    :   BytePerSector   =512;      SectorPerCluster=2;
                           ReservedSector  =1;        FATnumber       =2;
                           ROOTEntry       =112;      TotalSector     =1440;
                           SectorPerFAT    =3;        SectorPerTrack  =9;
                           break;
      case Disk_1440   :   BytePerSector   =512;      SectorPerCluster=1;
                           ReservedSector  =1;        FATnumber       =2;
                           ROOTEntry       =224;      TotalSector     =2879;
                           SectorPerFAT    =9;        SectorPerTrack  =18;
                           break;
      case Disk_320    :   BytePerSector   =512;      SectorPerCluster=2;
                           ReservedSector  =1;        FATnumber       =2;
                           ROOTEntry       =112;      TotalSector     =639;
                           SectorPerFAT    =1;        SectorPerTrack  =8;
                           break;
      case Disk_unknow :
      case Disk_2880   :
      default          :   Icannothandle();
   }
}
typedef struct
   {
/**/           char  jmp_exe_code[3];  /* jump to executable code            */
/**/           char  OEM_version[8];   /* OEM name and version               */
/**/  unsigned int   sectorbytes;      /* bytes per sector                   */
/**/  unsigned char  clustersize;      /* allocation unit size (in sector)   */
/**/  unsigned int   reservedsize;     /* reserved sectors (starting at 0)   */
/**/  unsigned char  FATnumber;        /* number of FAT's on disk            */
/**/  unsigned int   rootentry;        /* number of root directory entries   */
/**/  unsigned int   totalsector;      /* total sectors(0 if partition>32Mb) */
/**/  unsigned char  mediatype;        /* media descriptor byte              */
/**/  unsigned int   FATsize;          /* sectors per FAT                    */
/**/  unsigned int   tracksize;        /* sectors per track        (DOS 3.0+)*/
/**/  unsigned int   headnumber;       /* number of heads          (DOS 3.0+)*/
/**/  unsigned int   hiddensize;       /* number of hidden sectors (DOS 3.0+)*/
      unsigned long  totalsector2;     /* number of sectors if >32Mb (DOS 4+)*/
      unsigned char  reserved[2];      /* reserved                   (DOS 4+)*/
      unsigned char  drivenumber;      /* physical drive number      (DOS 4+)*/
      unsigned char  reserved2;        /* reserved                   (DOS 4+)*/
/**/  unsigned char  signature;        /* signature byte (29h)       (DOS 4+)*/
/**/  unsigned long  serialnumber;     /* volume serial number       (DOS 4+)*/
/**/           char  volumelabel[11];  /* volume label               (DOS 4+)*/
/**/           char  filesysID[8];     /* file system ID "FAT?????"  (DOS 4+)*/
   }  BOOTSEC;

void ReadBOOT(BOOTSEC *b,unsigned int bootoffset)
{
   if(fseek(ddifile,bootoffset*BytePerSector,SEEK_SET)!=0)
   {
      cout << "SEEK file ERROR !!" << endl;
      exit(1);
   }
   if(fread(b,sizeof(BOOTSEC),1,ddifile)!=1)
   {
      cout << "Read file ERROR !!" << endl;
      exit(1);
   }
}
void ResetParameter(BOOTSEC &b)
{
   BytePerSector   =b.sectorbytes ;
   SectorPerCluster=b.clustersize ;
   ReservedSector  =b.reservedsize;
   FATnumber       =b.FATnumber   ;
   ROOTEntry       =b.rootentry   ;
   TotalSector     =b.totalsector ;
   SectorPerFAT    =b.FATsize     ;
   SectorPerTrack  =b.tracksize   ;
}
void CompareFAT(unsigned int bootoffset)
{
   char *firstFAT,*secondFAT;
   firstFAT =new char[SectorPerFAT*BytePerSector];
   secondFAT=new char[SectorPerFAT*BytePerSector];
   fseek(ddifile,(bootoffset+1)*BytePerSector,SEEK_SET);
   fread(firstFAT,BytePerSector,SectorPerFAT,ddifile);
   fseek(ddifile,(bootoffset+1+SectorPerFAT)*BytePerSector,SEEK_SET);
   fread(secondFAT,BytePerSector,SectorPerFAT,ddifile);
   if(memcmp(firstFAT,secondFAT,SectorPerFAT*BytePerSector)!=0)
   {
      cout << "There are some different in 2 FAT !!" << endl;
      exit(1);
   }
   delete [] firstFAT;
   delete [] secondFAT;
}
void ConvertFAT(unsigned int bootoffset,int *fatchain,int totalfat)
{
   union
   {
      struct{  unsigned f1:12;            /* FAT 12 */
               unsigned f2:12;   }f;
      char a[3];
   };
   char *firstFAT;
   int i,j;
   firstFAT =new char[SectorPerFAT*BytePerSector];
   fseek(ddifile,(bootoffset+1)*BytePerSector,SEEK_SET);
   fread(firstFAT,BytePerSector,SectorPerFAT,ddifile);
   for(j=0;j<totalfat/2;j++)
   {
      for(i=0;i<3;i++) a[i]=firstFAT[i+j*3];
      fatchain[j*2]  =f.f1;
      fatchain[j*2+1]=f.f2;
//    showfat(fatchain[j*2]);
//    showfat(fatchain[j*2+1]);
   }
   delete [] firstFAT;
}

typedef struct                /* 76543210 File's attribute:  */
   {                          /*          read only        */
      unsigned readonly :1;   /*         hidden            */
      unsigned hidden   :1;   /*        system             */
      unsigned system   :1;   /*       volume label        */
      unsigned volume   :1;   /*      subdirectory         */
      unsigned directory:1;   /*     archive               */
      unsigned archive  :1;   /*   unused                 */
      unsigned unused   :2;
   }  FileAttr;
typedef struct         /*FEDCBA9876543210 Directory Time Format*/
   {                   /*             seconds            */
      unsigned sec:5;  /*                 (2 second increments)      */
      unsigned min:6;  /*       minutes (0-59)      */
      unsigned hour:5; /*  hours (0-23)         */
   }  FileTime;
typedef struct         /*FEDCBA9876543210 Directory Date Format*/
   {                   /*             Day (1-31)         */
      unsigned day:5;  /*         Month (1-12)        */
      unsigned mon:4;  /*  Year (less 1980)     */
      unsigned year:7;
   }  FileDate;
typedef struct
   {
      char filename[8];             /* Filename, left justified            */
      char fileext[3];              /* Filename extension, left justified  */
      union{FileAttr fa;            /* File's attribute                    */
            unsigned char fileattr;};
      char reserved[10];            /* Reserved by DOS                     */
      union{FileTime ft;            /* Time the file                       */
            unsigned int filetime;};
      union{FileDate fd;            /* Date the file                       */
            unsigned int filedate;};
      unsigned int  startcluster;   /* Starting cluster number of the file */
               long filesize;       /* File size in bytes                  */
   }  Directory;

long GetDiskState(char Driver,char Fflag)
{
   unsigned sectorpercluster;
   unsigned freeclusters;
   unsigned bytepersector;

   Driver=(Driver>='a' && Driver<='z')?Driver-'a'+'A':Driver; /* to Upper */
   Driver=(Driver>='A' && Driver<='Z')?Driver-'A'+1:0;   /* if not A-Z set 0 */
   asm{  mov   dl,Driver            /* 0->current, 1->A, 2->B, 3->C,... */
         mov   ah,036h
         int   21h
         mov   sectorpercluster,ax
         mov   freeclusters,bx
         mov   bytepersector,cx  };

   if(Fflag==1)
      return ((long)freeclusters*(long)sectorpercluster*(long)bytepersector);
   else
      return ((long)sectorpercluster*(long)bytepersector);
}
inline long GetDiskFreeSpace(char Driver)
{
   return GetDiskState(Driver,1);
}
void DealWithFilename(char *f,char *filename,char *fileext)
{
   char i,j;
   for(i=0;i<8;i++)f[i]=filename[i];
   f[i]='.';
   for(i=9;i<9+3;i++)f[i]=fileext[i-9];
}
char SetFileAttribute(char far *filename,unsigned attr)
{
   unsigned int fseg=FP_SEG(filename);
   unsigned int foff=FP_OFF(filename);
   char err=1;
   asm{  push  ds
         mov   ax,04301h
         mov   ds,fseg
         mov   dx,foff
         mov   cx,attr
         int   21h
         jc    errSetfileattr
         mov   err,0          };
errSetfileattr:
   asm{  pop   ds             };
   return err;
}
char SetFileDateTime(int fhandle,unsigned int fdate,unsigned int ftime)
{
   char err=1;
   asm{  mov   ah,057h
         mov   al,01h
         mov   bx,fhandle
         mov   cx,ftime
         mov   dx,fdate
         int   021h
         jc    errSetfiledatetime
         mov   err,0                };
errSetfiledatetime:
   return err;
}
void ReadFileCluster(void *buf,unsigned int clusterno)
{
   long offs=SectorPerTrack+1+FATnumber*SectorPerFAT
             +ROOTEntry/(BytePerSector/sizeof(Directory))
             +(clusterno-2)*SectorPerCluster;
   fseek(ddifile,offs*BytePerSector,SEEK_SET);
   fread(buf,BytePerSector,SectorPerCluster,ddifile);
}
void killspace(char *c)
{
   int slen=strlen(c);
   for(int i=slen-1;i>=0;i--)
      if(c[i]!=' ')
      {
         c[i+1]='\0';
         break;
      }
}
inline void addpathname(char *cp,char *fname,char *fext)
{
   strncat(cp,fname,8);killspace(cp);
   strcat(cp,".");
   strncat(cp,fext,3); killspace(cp);
   strcat(cp,"\\");
   int slen=strlen(cp);
   if(cp[slen-1-1]=='.')
      cp[slen-1-1]='\\', cp[slen-1]='\0';
}
void killpathname(char *cp)
{
   for(int j=strlen(cp)-1-1;j>=0;j--)
      if(cp[j]=='\\')
      {
         cp[j+1]='\0';
         break;
      }
}
inline void MakeDir(char *cp)
{
   int slen=strlen(cp);
   if(cp[slen-1]=='\\')
   {
      cp[slen-1]='\0';
      mkdir(cp);
      cp[slen-1]='\\'; cp[slen]='\0';
   }
   else
      mkdir(cp);
}
void showfilestatus(Directory &d)
{
/*-----------------------------------------------------------------------------
extact path name           filename ext  filesize   filedate filetime  attrib
=========================  ========.===  =========  ======== ========  ======
printf("%25.25s  %8.8s.%3.3s  %9.9s  %8.8s %8.8s %6.6s",
-----------------------------------------------------------------------------*/
   printf("\n%-25.25s  %8.8s.%3.3s  %9ld  %02d-%02d-%02d %02d:%02d:%02d  %s%s%s%s%s%s\r",
         (strlen(CurrentPATH)<=25)?CurrentPATH:(CurrentPATH+strlen(CurrentPATH)-25),
         d.filename, d.fileext, d.filesize,
         d.fd.mon, d.fd.day, d.fd.year+80,
         d.ft.hour, d.ft.min, d.ft.sec*2,
         (d.fa.archive  ==1)?"A":" ",
         (d.fa.readonly ==1)?"R":" ",
         (d.fa.system   ==1)?"S":" ",
         (d.fa.hidden   ==1)?"H":" ",
         (d.fa.directory==1)?"D":" ",
         (d.fa.volume   ==1)?"V":" " );
}
void showfilesize(int fs)
{
   static long ffs;
   if(fs==0)ffs=0; else ffs+=fs;
   gotoxy(42,wherey());printf("%9ld",ffs);
}
void SaveThisFile(Directory &d)
{
   if(d.fa.directory==1)return;
   if(d.fa.volume==1)return;

   strcpy(CurrentPATHfilename,CurrentPATH);
   addpathname(CurrentPATHfilename,d.filename,d.fileext);
   CurrentPATHfilename[strlen(CurrentPATHfilename)-1]='\0';

   if(!FlagofOverwrite)
      if(access(CurrentPATHfilename,0)==0)
      {
         int ch=0;
         printf("\r%79.79s\r"," ");
         printf("%-33.33s  Already exist ! (S)kip (O)verwrite (A)ll ? ",CurrentPATHfilename);
         while(bioskey(1)!=0) bioskey(0);
         while(ch==0)
            switch(bioskey(0))
            {
               case N_S: case S_S: putch('S'); ch=1; return;
               case N_O: case S_O: putch('O'); ch=1; break;
               case N_A: case S_A: putch('A'); ch=1; FlagofOverwrite=1; break;
               default:                        ch=0; break;
            }
         printf("\r%79.79s\r"," ");gotoxy(1,wherey()-1);
         showfilestatus(d);
      }

   unsigned int stcluster=d.startcluster;
   int thishandle;
   thishandle=open(CurrentPATHfilename,O_WRONLY|O_CREAT|O_EXCL|O_BINARY,S_IWRITE);
   showfilesize(0);
   while(1)
   {
      ReadFileCluster(filebuffer,stcluster);
      if(FATchain[stcluster]>0xFF0)          /* test if <EOF> */
      {
         write(thishandle,filebuffer,d.filesize%(BytePerSector*SectorPerCluster));
         showfilesize(d.filesize%(BytePerSector*SectorPerCluster));
         break;
      }
      else
         write(thishandle,filebuffer,BytePerSector*SectorPerCluster);
      showfilesize(BytePerSector*SectorPerCluster);
      stcluster=FATchain[stcluster];
   }
   SetFileDateTime(thishandle,d.filedate,d.filetime);
   close(thishandle);
   SetFileAttribute(CurrentPATHfilename,d.fileattr);
}
void showdir(Directory *d,int numberofd)
{
   void ShowDirectory(unsigned int);
   int i;
   for(i=0;i<numberofd;i++)
   {
      if(d[i].filename[0]=='\0')break;          /* End of Entry */
      if(d[i].filename[0]=='')continue;        /* file has been deleted */
      showfilestatus(d[i]);
      if(!FlagofCheckonly)SaveThisFile(d[i]);
      if(d[i].fa.directory==1 && d[i].filename[0]!='.' && d[i].filename[0]!='')
      {
         addpathname(CurrentPATH,d[i].filename,d[i].fileext);
         if(!FlagofCheckonly)
         {
            struct date dn,df;
            struct time tn,tf;
            df.da_year=d[i].fd.year+1980;df.da_mon=d[i].fd.mon;df.da_day =d[i].fd.day;
            tf.ti_min=d[i].ft.min;tf.ti_hour=d[i].ft.hour;tf.ti_sec=d[i].ft.sec*2;tf.ti_hund=0;
            getdate(&dn);setdate(&df);
            gettime(&tn);settime(&tf);
            MakeDir(CurrentPATH);
            settime(&tn);setdate(&dn);
         }
         ShowDirectory(d[i].startcluster);
         killpathname(CurrentPATH);
      }
   }
}
void ShowDirectory(unsigned int startclus)
{
   int i=BytePerSector*SectorPerCluster/sizeof(Directory);
   Directory *d=new Directory[i];
   while(1)
   {
      ReadFileCluster((void *)d,startclus);
      showdir(d,i);
      if(FATchain[startclus]>0xFF0) break;         /* test if <EOF> */
      startclus=FATchain[startclus];
   }
   delete [] d;
}
void ShowROOT(unsigned int bootoffset)
{
   Directory *d=new Directory[ROOTEntry];
   fseek(ddifile,(bootoffset+1+SectorPerFAT*FATnumber)*BytePerSector,SEEK_SET);
   fread(d,sizeof(Directory),ROOTEntry,ddifile);
   showdir(d,ROOTEntry);
   delete [] d;
}

enum PROCESS { ABORT, CONTINUE };
int  NormalControlBreak(void)
{
   printf("\nUser Break!!\n");
   exit(0);
   return (ABORT);               /* quit program */
/* return (CONTINUE);            /* continue program */  */
}

char CheckChineseSystem(void)
{
   asm{  mov   ax,09100h
         mov   bx,0
         int   10h

         cmp   bx,0
         je    NotInET

         mov   ah,80h
         mov   al,'N'
         int   10h      };

   return 1;

NotInET:
   return 0;
}

char *Eusage="USAGE :  %s  [options]  DDIfile[.ddi]  [[drive:]\\path]\n"
             "options:\n"
             "\t/o\tOverwrite if file already exist\n"
             "\t/c\tCheck only and show the files in DDI\n"
             "\t/n\tNot to compare two FATs\n"
             "\t/s\tuse Standard floppy disk format\n"
             "\t/d:[no]\tDisk size contain in DDI file\n"
                 "\t\tno:\t1 --> 360  Kb\t2 --> 1200 Kb\n"
                    "\t\t\t3 --> 720  Kb\t4 --> 1440 Kb\n"
                    "\t\t\t5 -->*2880 Kb\t6 --> 320  Kb\n\n"
             "EXAMPLE:\n"
             "\t%s test.ddi\n"
             "\t%s test.ddi c:\\test\n"
             "\t%s /c test.ddi\n"
             "\t%s /s /t /d:2 test.ddi c:\\test\n";

char *Cusage="ΪkG %s  [ﶵ]  DDIɦW[.ddi]  [[ϺоW:]\\ؿ]\n\n"
             "ﶵG"
             "\t/o\t\tɮפwsbN|\g\n"
             "\t/c\t\tuˬdèqX DDI ɦW\n"
             "\t/n\t\t FAT O_ۦP\n"
             "\t/s\t\tϥμзǳnЪ榡\n"
             "\t/d:[N]\tDDI ɤҧtƪjp\n"
                 "\t\t\tN:\t1 --> 360  Kb\t2 --> 1200 Kb\n"
                      "\t\t\t\t3 --> 720  Kb\t4 --> 1440 Kb\n"
                      "\t\t\t\t5 -->*2880 Kb\t6 --> 320  Kb\n\n"
             "dҡG"
             "\t%s  test.ddi\n"
             "\t%s  test.ddi c:\\test\n"
             "\t%s  /c test.ddi\n"
             "\t%s  /n /s /d:2 test.ddi c:\\test\n";

char *pline  ="-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-\n";
char *pline2 ="=========================";
char *Eauther="UnDDI         Version 0.1           Jul.12,1994             by : Lee Dong-Liang\n";
char *Cauther="UnDDI        0.1               KQT~CQG                 byG ɼ\n";
char *Eheader="extact path name           filename ext  filesize   filedate filetime  attrib\n";
char *Cheader="extact path name           filename ext  filesize   filedate filetime  attrib\n";


void ShowTitle(void)
{
   printf(pline);
   if(inchinesesystem)printf(Cauther);
   else               printf(Eauther);
   printf(pline);
}
void ShowHeader(void)
{
/*-----------------------------------------------------------------------------
extact path name           filename ext  filesize   filedate filetime  attrib
=========================  ========.===  =========  ======== ========  ======
-----------------------------------------------------------------------------*/
   if(inchinesesystem)printf(Cheader);
   else               printf(Eheader);
   printf("%25.25s  %8.8s.%3.3s  %9.9s  %8.8s %8.8s  %6.6s",pline2,pline2,pline2,pline2,pline2,pline2,pline2);
   window(1,wherey(),80,24);
}
void Help(char *ex)
{
   int i;
   for(i=strlen(ex);i>=0;i--)
      if(ex[i]=='\\'){i++; break;}
   char *s=&ex[i];
   if(inchinesesystem)printf(Cusage,s,s,s,s,s);
   else               printf(Eusage,s,s,s,s,s);
   exit(1);
}
void FlagSetting(int argc_,char *argv_[])
{
   if(argc_<2)  Help(argv_[0]);
   int j=0;
   for(int i=1;i<argc_;i++)
   {
      if(argv_[i][0]=='/')       /* This is a option */
      {
         switch(argv_[i][1])
         {
            case 'O': case 'o':  FlagofOverwrite=1;   break;
            case 'C': case 'c':  FlagofCheckonly=1;   break;
            case 'N': case 'n':  FlagofNocmpFAT=1;    break;
            case 'S': case 's':  FlagofStdformat=1;   break;
            case 'D': case 'd':  if(argv_[i][2]==':') DDIDisksize=atoi(&argv_[i][3]);
                                 if(DDIDisksize<1 || DDIDisksize>6)DDIDisksize=0; break;
            default :            Help(argv_[0]);
         }
      }
      else                       /* This is a option */
      {
         j++;
         if(j==1)
         {
            strncpy(DDIfilename,argv_[i],79),DDIfilename[79]='\0';
            if(DDIfilename[strlen(DDIfilename)-4]!='.')strcat(DDIfilename,".DDI");
         }
         if(j==2)
         {
            strncpy(CurrentPATH,argv_[i],299),CurrentPATH[299]='\0';
            if(CurrentPATH[strlen(CurrentPATH)-1]!='\\')strcat(CurrentPATH,"\\");
         }
         if(j>=3) Help(argv_[0]);
      }
   }
   if(j<2)
   {
      strcpy(CurrentPATH,"D:\\");CurrentPATH[0]='A'+getdisk();getcurdir(0,CurrentPATH+3);
      if(CurrentPATH[strlen(CurrentPATH)-1]!='\\')strcat(CurrentPATH,"\\");
   }
}

void main(int argc,char *argv[])
{
   DISKTYPE d;
   BOOTSEC b;
   clrscr();
   ctrlbrk(NormalControlBreak);                 /* Set Ctrl-Break function  */

   inchinesesystem=CheckChineseSystem();
   ShowTitle();

   DDIfilename=new char[80];
   CurrentPATH=new char[300];
   CurrentPATHfilename=new char[320];
   FlagSetting(argc,argv);

   if((ddifile=fopen(DDIfilename,"rb"))==NULL)
   {
      cout << "Cannot Open DDI file !!" << endl;
      exit(1);
   }
   if(DDIDisksize==0) d=GetDDIfiletype();
   else               d=(DISKTYPE)DDIDisksize;
   printf("Disk Size is (Kb) : %d\n",(d==Disk_360 )? 360:((d==Disk_1200)?1200:
                                    ((d==Disk_720 )? 720:((d==Disk_1440)?1440:
                                    ((d==Disk_2880)?2880:((d==Disk_320 )? 320:0))))) );
   ShowHeader();
   SetDefault(d);
   ReadBOOT(&b,SectorPerTrack);
// ShowBOOT(b);
   if(FlagofStdformat==0) ResetParameter(b);
   if(FlagofNocmpFAT==0)  CompareFAT(SectorPerTrack);

   totalFAT=SectorPerFAT*BytePerSector*2/3;
   FATchain=new int[totalFAT];
   ConvertFAT(SectorPerTrack,FATchain,totalFAT);

   filebuffer=new char[SectorPerCluster*BytePerSector];

   ShowROOT(SectorPerTrack);

   delete [] filebuffer;
   delete [] CurrentPATHfilename;
   delete [] CurrentPATH;
   delete [] DDIfilename;
   delete [] FATchain;
   fclose(ddifile);
}

/*-----------------------------------------------------------------------------
                       Common Floppy Disk Formats
               Supporting                SectorsĿ   Entries
          Disk    DOS              per  per  per   per   per   Total
          Type   Vers  Sides Trks  TRK  FAT  DIR Cluster DIR  Sectors
        
        5 160k  (1.0)   1   (40)   8    1    4     1     64     320
        5 320k  (1.1)   2   (40)   8    1    7     2    112     640
        5 180k  (2.0)   1   (40)   9    2    4     1     64     360
        5 360k  (2.0)   2   (40)   9    2    7     2    112     720
        5 1.2M  (3.0)   2   (80)  15    7   14     1    224    2400
        3 720k  (3.2)   2   (80)   9    3    7     2    112    1440
        3 1.44M (3.3)   2   (80)  18    9   14     1    224    2880

Area  1440 Kb total  1200 Kb total  720  Kb total  360  Kb total
====  ======= =====  ======= =====  ======= =====  ======= =====
BOOT   0          1   0          1   0          1   0          1
FAT1   1-9        9   1-7        7   1-5        5   1-3        3
FAT2  10-18       9   8-14       7   6-10       5   4-6        3
ROOT  19-32      14  15-28      14  11-17       7   7-13       7
DATA  33-2879  2847  29-2399  2371  18-1439  1422  14-719    706
====  ======= =====  ======= =====  ======= =====  ======= =====
Diff  +31            +27            +16            +12  Sector = Cluster + Diff

               1440 Kb  1200 Kb  720  Kb  360  Kb
=============  =======  =======  =======  =======
DDIoffset           18       15        9        9  = Sector per Track

DDI disk size : offset 000A   byte
               360  Kb  1200 Kb  720  Kb  1440 Kb  2880 Kb  320  Kb
=============  =======  =======  =======  =======  =======  =======
ID character    = 0x1   = 0x2   = 0x3   = 0x4   = 0x5   = 0x6
-----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
   printf("%8.8s  %3.3s  %9ld  %02d-%02d-%02d  %02d:%02d:%02d  %7d %3s %3s %3s %3s %3s %3s\n",
         d[i].filename, d[i].fileext, d[i].filesize,
         d[i].fd.mon, d[i].fd.day, d[i].fd.year+80,
         d[i].ft.hour, d[i].ft.min, d[i].ft.sec*2, d[i].startcluster,
         (d[i].fa.archive  ==1)?"Arc":"",
         (d[i].fa.readonly ==1)?"R/O":"",
         (d[i].fa.system   ==1)?"Sys":"",
         (d[i].fa.hidden   ==1)?"Hid":"",
         (d[i].fa.directory==1)?"Dir":"",
         (d[i].fa.volume   ==1)?"Vol":"" );

void showfat(unsigned ff)
{
   if(ff==0)                 { cout << "free "; return; }
   if(ff>=0xFF1 && ff<=0xFF7){ cout << "sBAD "; return; }
   if(ff>=0xFF8 && ff<=0xFFE){ cout << "nEND "; return; }
   if(ff>=0xFFF)             { cout << "sEND "; return; }
   printf("%4d ",ff);
}
void ShowBOOT(BOOTSEC b)
{
// printf("                           OEM ID: %8.8s\n"    ,b.OEM_version);
   printf("                 Bytes per sector: %u\n"       ,b.sectorbytes);
   printf("              Sectors per cluster: %u\n"       ,b.clustersize);
   printf("    Reserved sectors at beginning: %u\n"       ,b.reservedsize);
   printf("                       FAT Copies: %u\n"       ,b.FATnumber);
   printf("           Root directory entries: %u\n"       ,b.rootentry);
   printf("            Total sectors on disk: %04X Hex\n" ,b.totalsector);
// printf("            Media descriptor byte: %02X Hex\n" ,b.mediatype);
   printf("                  Sectors per FAT: %u\n"       ,b.FATsize);
   printf("                Sectors per track: %u\n"       ,b.tracksize);
// printf("                            Sides: %u\n"       ,b.headnumber);
// printf("           Special hidden sectors: %u\n"       ,b.hiddensize);
// printf("      Big total number of sectors: %08lX Hex\n",b.totalsector2);
// printf("            Physical drive number: %02X\n"     ,b.drivenumber);
// printf("   Extended Boot Record Signature: %02X Hex\n" ,b.signature);
// printf("             Volume Serial Number: %08lX Hex\n",b.serialnumber);
// printf("                     Volume Label: %11.11s\n"  ,b.volumelabel);
// printf("                   File System ID: %8.8s\n"    ,b.filesysID);
}
-----------------------------------------------------------------------------*/
